/* Copyright (C) 1989 Aladdin Enterprises.  All rights reserved.
   Distributed by Free Software Foundation, Inc.

This file is part of Ghostscript.

Ghostscript is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the Ghostscript General Public License for full details.

Everyone is granted permission to copy, modify and redistribute
Ghostscript, but only under the conditions described in the Ghostscript
General Public License.  A copy of this license is supposed to have been
given to you along with Ghostscript so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.  */

/* zdevice.c */
/* Device-related operators for GhostScript */
#include <math.h>			/* for fabs */
#include "ghost.h"
#include "errors.h"
#include "oper.h"
#include "state.h"
#include "gsmatrix.h"
#include "gsstate.h"
#include "store.h"

/* showpage */
int
zshowpage(register ref *op)
{	/* No distinction between showpage and copypage is supported. */
	return gs_copypage(igs);
}

/* copypage */
int
zcopypage(register ref *op)
{	return gs_copypage(igs);
}

/* .framedevice */
int
zframedevice(register ref *op)
{	return common_device(op, gs_framedevice);
}

/* Common code for banddevice and framedevice */
static int
common_device(register ref *op,
  int (*dproc)(P4(gs_state *, gs_matrix *, int, int)))
{	gs_matrix imat;
	int code;
	check_type(op[-1], t_integer);
	check_type(op[-2], t_integer);
	if (	(ulong)(op[-1].value.intval) > (uint)(-1) >> 1 ||
		(ulong)(op[-2].value.intval) > (uint)(-1) >> 1
	   ) return e_rangecheck;
	if ( (code = read_matrix(op - 3, &imat)) < 0 ) return code;
	check_proc(*op);
	/* Everything OK, install device */
	code = (*dproc)(igs, &imat,
		(int)op[-2].value.intval, (int)op[-1].value.intval);
	istate.show_proc = *op;
	pop(4);
	return 0;
}

/* .nulldevice */
int
znulldevice(register ref *op)
{	gs_nulldevice(igs);
	return 0;
}

/* currentdevice */
int
zcurrentdevice(register ref *op)
{	int code = write_matrix(op);
	int width, height;
	if ( code < 0 ) return code;
	gs_currentdevice(igs, (gs_matrix *)op->value.refs, &width, &height);
	push(3);
	make_int(op - 2, width);
	make_int(op - 1, height);
	*op = istate.show_proc;
	return 0;
}

/* ------ Initialization procedure ------ */

void
zdevice_op_init()
{	static op_def my_defs[] = {
		{"0copypage", zcopypage},
		{"1currentdevice", zcurrentdevice},
		{"4.framedevice", zframedevice},
		{"0.nulldevice", znulldevice},
		{"0showpage", zshowpage},
		op_def_end
	};
	z_op_init(my_defs);
}
